package jp.co.intra_mart.tools.bm5.csv2xml.export;

import javax.xml.parsers.ParserConfigurationException;

import jp.co.intra_mart.common.aid.jdk.javax.xml.XmlNode;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.Account;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.ApplicationRole;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.Role;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.UserMap;
import jp.co.intra_mart.tools.bm5.csv2xml.util.XmlUtil;

import org.w3c.dom.Document;

/**
 * AJEgpXMLhLg쐬NX
 * 
 * @since 5.1
 * @author INTRAMART
 * @version 1.0
 */
public class AccountExport extends Export {

	private static final String ACCOUNTS = "accounts";

	private static final String ACCOUNT = "account";

	private static final String USER_ID = "user-id";

	private static final String LOCALE = "locale";

	private static final String ENCODING = "encoding";

	private static final String PASSWORD = "password";

	private static final String MOBILE_PASSWORD = "mobile-password";

	private static final String NOTE = "note";

	private static final String VALID_START_DATE = "valid-start-date";

	private static final String VALID_END_DATE = "valid-end-date";

	private static final String GO_OUT = "go-out";

	private static final String MAIN_PAGE_PATTERN = "main-page-pattern";

	private static final String COLOR_PATTERN = "color-pattern";

	private static final String LICENSE = "license";

	private static final String ROLE = "role";

	// private static final String FAVORITE = "favorite";

	private static final String APP_ROLE = "app-role";

	private static final String ROLE_ID = "role-id";

	private static final String ROLE_VALID_START_DATE = "role-valid-start-date";

	private static final String ROLE_VALID_END_DATE = "role-valid-end-date";

	/**
	 * [UfAJEgXMLhLg𐶐܂B
	 * 
	 * @param modelMap [Uf
	 * @return AJEgXMLhLg
	 */
	public Document createDocument(Object modelMap)
			throws ParserConfigurationException {

		// f̎擾
		UserMap userMap = (UserMap) modelMap;
		Account[] accounts = userMap.getAccounts();

		// hLg̐
		Document doc = XmlUtil.getDocument();

		// <accounts>
		XmlNode accountsNode = new XmlNode(doc.createElement(ACCOUNTS));
		doc.appendChild(accountsNode.getNode());

		for (int i = 0; i < accounts.length; i++) {

			Account accountModel = accounts[i];

			// <account>
			XmlNode accountNode = accountsNode.addNode(ACCOUNT);

			// <user-id>
			addNode(accountNode, USER_ID, accountModel.getUserCode());

			// <locale>
			addNode(accountNode, LOCALE, accountModel.getLocale());

			// <encoding>
			addNode(accountNode, ENCODING, accountModel.getEncoding());

			// <password>
			addNode(accountNode, PASSWORD, accountModel.getPassword());

			// <mobile-password>
			addNode(accountNode, MOBILE_PASSWORD, accountModel
					.getMobilePassword());

			// <note>
			addNode(accountNode, NOTE, accountModel.getNotes());

			// <valid-start-date>
			addNode(accountNode, VALID_START_DATE, accountModel.getTerm()
					.getActionvationString());

			// <valid-end-date>
			addNode(accountNode, VALID_END_DATE, accountModel.getTerm()
					.getExpirationString());

			// <go-out>
			addNode(accountNode, GO_OUT, accountModel.getGoOut());

			// <main-page-pattern>
			addNode(accountNode, MAIN_PAGE_PATTERN, accountModel
					.getMainPageParttern());

			// <color-pattern>
			addNode(accountNode, COLOR_PATTERN, accountModel.getColorParttern());

			// <license>
			addNode(accountNode, LICENSE, accountModel.getLicense());

			// <role>
			this.createRole(accountNode, accountModel);

			// <favorite>
			// accountNode.addNode();

			// <app-role>
			for (int k = 0; k < accountModel.getApplicationRoles().length; k++) {
				ApplicationRole appRole = accountModel.getApplicationRoles()[k];
				addNode(accountNode, APP_ROLE, appRole.getApplicationRole());
			}
		}

		return doc;
	}

	/**
	 * role^O쐬܂B
	 * 
	 * @param accountNode
	 * @param accountModel
	 */
	private void createRole(XmlNode accountNode, Account accountModel) {
		Role[] roles = accountModel.getRoles();

		if (roles.length == 0) return;
		
		for (int k = 0; k < roles.length; k++) {
			
			XmlNode roleNode = accountNode.addNode(ROLE);

			Role roleModel = roles[k];

			// <role-id>
			addNode(roleNode, ROLE_ID, roleModel.getRoleId());

			// <role-valid-start-date>
			addNode(roleNode, ROLE_VALID_START_DATE, roleModel.getTerm()
					.getActionvationString());

			// <role-valid-end-date>
			addNode(roleNode, ROLE_VALID_END_DATE, roleModel.getTerm()
					.getExpirationString());
		}
	}
}
