package jp.co.intra_mart.tools.bm5.csv2xml.export;

import javax.xml.parsers.ParserConfigurationException;

import jp.co.intra_mart.common.aid.jdk.javax.xml.XmlNode;
import jp.co.intra_mart.tools.bm5.csv2xml.models.common.Term;
import jp.co.intra_mart.tools.bm5.csv2xml.models.post.CompanyPost;
import jp.co.intra_mart.tools.bm5.csv2xml.models.post.CompanyPostMap;
import jp.co.intra_mart.tools.bm5.csv2xml.models.post.Post;
import jp.co.intra_mart.tools.bm5.csv2xml.models.post.PostTerm;
import jp.co.intra_mart.tools.bm5.csv2xml.models.post.PostTermI18n;
import jp.co.intra_mart.tools.bm5.csv2xml.util.XmlUtil;

import org.w3c.dom.Document;

/**
 * EpXMLhLg쐬NX
 * 
 * @since 5.1
 * @author INTRAMART
 * @version 1.0
 */
public class PostExport extends Export {

	
	private final static String COMPANY_POST_DATA = "company-post-data";
	
	// <company-post-data>
	private final static String COMPANY = "company";
	
	// <company>
	private final static String COMPANY_CODE = "company-code";
	private final static String COMPANY_POST = "company-post";
	
	// <company-post>
	private final static String POST_CODE = "post-code";
	private final static String NOTES = "notes";
	private final static String SORT_KEY = "sort-key";
	private final static String COMPANY_POST_TERM = "company-post-term";
	
	// <company-post-term>
	private final static String START_DATE = "start-date";
	private final static String END_DATE = "end-date";
	private final static String COMPANY_POST_TERM_I18N = "company-post-term-i18n";
	
	// <company-post-term-i18n>
	private final static String LOCALE = "locale";
	private final static String NAME = "name";
	
	/**
	 * EfEXMLhLg𐶐܂B
	 * 
	 * @param modelMap Ef
	 * @return EXMLhLg
	 */
	public Document createDocument(Object modelMap)
		throws ParserConfigurationException{
		
		// f̎擾
		CompanyPostMap postMap = (CompanyPostMap) modelMap;

		// hLg̐
		Document doc = XmlUtil.getDocument();

		// <company-post-data>
		XmlNode postDataNode = new XmlNode(doc.createElement(COMPANY_POST_DATA));
		doc.appendChild(postDataNode.getNode());
		
		CompanyPost[] companies = postMap.getCompanyPosts();
		
		for(int i = 0; i < companies.length; i++) {
			
			CompanyPost company = companies[i];
			
			// <company>
			XmlNode companyNode = postDataNode.addNode(COMPANY);
			
			// <company-code>
			addNode(companyNode, COMPANY_CODE, company.getCompanyCode());
			
			// <company-post>
			createCompanyPost(companyNode, company);
		}
		
		return doc;
	}
	
	private void createCompanyPost(XmlNode companyNode, CompanyPost company) {
		
		Post[] posts = company.getPosts();
		
		for(int i = 0; i < posts.length; i++) {
		
			Post post = posts[i];
			
			// <company-post>
			XmlNode companyPostNode = companyNode.addNode(COMPANY_POST);
			
			// <post-code>
			addNode(companyPostNode, POST_CODE, post.getPostCode());
			
			// <notes>
			addNode(companyPostNode, NOTES, post.getNotes());
			
			// <sort-key>
			addNode(companyPostNode, SORT_KEY, post.getSortKey());
			
			PostTerm[] postTerms = post.getTerminables();
			
			for (int k = 0; k < postTerms.length; k++) {
				
                PostTerm postTerm = postTerms[k];
				
                Term term = postTerm.getTerm();
				
				// <company-post-term>
				XmlNode companyPostTermNode = companyPostNode.addNode(COMPANY_POST_TERM);
				
				// <start-date>
				addNode(companyPostTermNode, START_DATE, term.getActionvationString());
				
				// <end-date>
				addNode(companyPostTermNode, END_DATE, term.getExpirationString());
				
                PostTermI18n[] postTermi18n = postTerm.getTerminableInternationals();
                for (int l = 0; l < postTermi18n.length; l++) {
                    
                    // <company-post-term-i18n>
                    XmlNode companyPostTermI18nNode = companyPostTermNode.addNode(COMPANY_POST_TERM_I18N);
                    
                    // <locale>
                    addNode(companyPostTermI18nNode, LOCALE, postTermi18n[l].getLocale());
                    
                    // <name>
                    addNode(companyPostTermI18nNode, NAME, postTermi18n[l].getName());
                }
			}
		}
	}
}
