package jp.co.intra_mart.tools.bm5.csv2xml.export;

import javax.xml.parsers.ParserConfigurationException;

import jp.co.intra_mart.common.aid.jdk.javax.xml.XmlNode;
import jp.co.intra_mart.tools.bm5.csv2xml.models.common.Term;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.CompanyMember;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.DepartmentMember;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.MainMember;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.PublicGroupMember;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.PublicGroupSetMember;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.User;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.UserMap;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.UserTerm;
import jp.co.intra_mart.tools.bm5.csv2xml.models.user.UserTermI18n;
import jp.co.intra_mart.tools.bm5.csv2xml.util.XmlUtil;

import org.w3c.dom.Document;

/**
 * vt@CpXMLhLg쐬NX
 * 
 * @since 5.1
 * @author INTRAMART
 * @version 1.0
 */
public class ProfileExport extends Export {

	private static final String PROFILE_DATA = "profile-data";

	// <profile-data>
	private static final String PROFILE = "profile";

	// <profile>
	private static final String USER_CODE = "user-code";

	private static final String NOTES = "notes";

	private static final String SORT_KEY = "sort-key";

	private static final String PROFILE_TERM = "profile-term";

	private static final String COMPANY_MEMBER = "company-member";

	private static final String PROFILE_MAIN = "profile-main";

	private static final String PUBLIC_GROUP_SET_MEMBER = "public-group-set-member";

	// <company-member>
	private static final String COMPANY_CODE = "company-code";

	private static final String DEPARTMENT_MEMBER = "department-member";

	// <department-member>
	private static final String DEPARTMENT_CODE = "department-code";

	// private static final String SORT_KEY = "sort-key"; // ^Ôŏȗ
	private static final String DEPARTMENT_MEMBER_TERM = "department-member-term";

	// <department-member-term>
	// private static final String START_DATE = "start-date"; // ^Ôŏȗ
	// private static final String END_DATE = "end-date"; // ^Ôŏȗ
	private static final String POST_CODE = "post-code";

	// <public-group-set-member>
	private static final String PUBLIC_GROUP_SET_CODE = "public-group-set-code";

	private static final String PUBLIC_GROUP_MEMBER = "public-group-member";

	// <public-group-member>
	private static final String PUBLIC_GROUP_CODE = "public-group-code";

	// private static final String SORT_KEY = "sort-key"; // ^Ôŏȗ
	private static final String PUBLIC_GROUP_MEMBER_TERM = "public-group-member-term";

	// <public-group-member-term>
	// private static final String START_DATE = "start-date"; // ^Ôŏȗ

	// private static final String END_DATE = "end-date"; // ^Ôŏȗ

	// <profile-main>
	private static final String MAIN_TERM = "main-term";

	// <main-term>
	// private static final String START_DATE = "start-date"; // ^Ôŏȗ

	// private static final String END_DATE = "end-date"; // ^Ôŏȗ

	// private static final String COMPANY_CODE = "company-code"; // ^Ôŏȗ

	// private static final String DEPARTMENT_CODE = "department-code"; //
	// ^Ôŏȗ

	// <profile-term>
	private static final String START_DATE = "start-date";

	private static final String END_DATE = "end-date";

	private static final String TELEPHONE_NUMBER = "telephone-number";

	private static final String MOBILE_NUMBER = "mobile-number";

	private static final String FAX_NUMBER = "fax-number";

	private static final String EXTENSION_NUMBER = "extension-number";

	private static final String EXTENSION_FAX_NUMBER = "extension-fax-number";

	private static final String COUNTRY_CODE = "country-code";

	private static final String ZIP_CODE = "zip-code";

	private static final String EMAIL_ADDRESS1 = "email-address1";

	private static final String EMAIL_ADDRESS2 = "email-address2";

	private static final String MOBILE_EMAIL_ADDRESS = "mobile-email-address";

	private static final String URL = "url";

	private static final String PROFILE_TERM_I18N = "profile-term-i18n";

	// <profile-term-i18n>
	private static final String LOCALE = "locale";

	private static final String USER_NAME = "user-name";

	private static final String USER_NAME_SYLLABARY = "user-name-syllabary";

	private static final String USER_NAME_ENGLISH = "user-name-english";

	private static final String ADDRESS = "address";

	/**
	 * [Ufvt@CXMLhLg𐶐܂B
	 * 
	 * @param modelMap
	 *            [Uf
	 * @return vt@CXMLhLg
	 */
	public Document createDocument(Object modelMap)
			throws ParserConfigurationException {
		// f̎擾
		UserMap userMap = (UserMap) modelMap;
		User[] users = userMap.getUsers();

		// hLg̐
		Document doc = XmlUtil.getDocument();

		// <profile-data>
		XmlNode profileDataNode = new XmlNode(doc.createElement(PROFILE_DATA));
		doc.appendChild(profileDataNode.getNode());

		for (int i = 0; i < users.length; i++) {

			User userModel = users[i];

			// <profile>
			XmlNode profileNode = profileDataNode.addNode(PROFILE);

			// <user-code>
			addNode(profileNode, USER_CODE, userModel.getUserCode());

			// <notes>
			addNode(profileNode, NOTES, userModel.getNotes());

			// <sort-key>
			addNode(profileNode, SORT_KEY, userModel.getSortKey());

			// <profile-term>
			createProfileTerm(profileNode, userModel);

			// <company-member>
			createCompanyMember(profileNode, userModel);

			// <profile-main>
			createProfileMain(profileNode, userModel);

			// <public-group-set-member>
			createPublicGroupSetMember(profileNode, userModel);
		}

		return doc;
	}

	public void createProfileTerm(XmlNode profileNode, User userModel) {

		for (int i = 0; i < userModel.getUserTerms().length; i++) {

			UserTerm userTerm = userModel.getUserTerms()[i];

			// <profile-term>
			XmlNode profileTermNode = profileNode.addNode(PROFILE_TERM);

			// <start-date>
			addNode(profileTermNode, START_DATE, userTerm.getTerm()
					.getActionvationString());

			// <end-date>
			addNode(profileTermNode, END_DATE, userTerm.getTerm()
					.getExpirationString());

			// <telephone-number>
			addNode(profileTermNode, TELEPHONE_NUMBER, userTerm
					.getTelephoneNumber());

			// <mobile-number>
			addNode(profileTermNode, MOBILE_NUMBER, userTerm.getMobileNumber());

			// <fax-number>
			addNode(profileTermNode, FAX_NUMBER, userTerm.getFaxNumber());

			// <extension-number>
			addNode(profileTermNode, EXTENSION_NUMBER, userTerm
					.getExtensionNumber());

			// <extension-fax-number>
			addNode(profileTermNode, EXTENSION_FAX_NUMBER, userTerm
					.getExtensionFaxNumber());

			// <country-code>
			addNode(profileTermNode, COUNTRY_CODE, userTerm.getCountryCode());

			// <zip-code>
			addNode(profileTermNode, ZIP_CODE, userTerm.getZipCode());

			// <email-address1>
			addNode(profileTermNode, EMAIL_ADDRESS1, userTerm
					.getEmailAddress1());

			// <email-address2>
			addNode(profileTermNode, EMAIL_ADDRESS2, userTerm
					.getEmailAddress2());

			// <mobile-email-address>
			addNode(profileTermNode, MOBILE_EMAIL_ADDRESS, userTerm
					.getMobileEmailAddress());

			// <url>
			addNode(profileTermNode, URL, userTerm.getUrl());

			// <profile-term-i18n>
			UserTermI18n[] userTermI18n = userTerm
					.getTerminableInternationals();
			for (int k = 0; k < userTermI18n.length; k++) {

				UserTermI18n i18nModel = userTermI18n[k];

				XmlNode i18nNode = profileTermNode.addNode(PROFILE_TERM_I18N);

				// <locale>
				addNode(i18nNode, LOCALE, i18nModel.getLocale());

				// <user-name>
				addNode(i18nNode, USER_NAME, i18nModel.getName());

				// <user-name-syllabary>
				addNode(i18nNode, USER_NAME_SYLLABARY, i18nModel
						.getNameSyllabary());

				// <user-name-english>
				addNode(i18nNode, USER_NAME_ENGLISH, i18nModel.getNameEnglish());

				// <address>
				addNode(i18nNode, ADDRESS, i18nModel.getAddress());
			}
		}
	}

	public void createProfileMain(XmlNode profileNode, User userModel) {

		MainMember[] mainMembers = userModel.getMainMembers();

		if (mainMembers.length == 0)
			return;

		// <profile-main>
		XmlNode profileMainNode = profileNode.addNode(PROFILE_MAIN);

		for (int i = 0; i < mainMembers.length; i++) {

			MainMember mainMember = mainMembers[i];

			// <main-term>
			XmlNode mainTermNode = profileMainNode.addNode(MAIN_TERM);

			// <start-date>
			addNode(mainTermNode, START_DATE, mainMember.getTerm()
					.getActionvationString());

			// <end-date>
			addNode(mainTermNode, END_DATE, mainMember.getTerm()
					.getExpirationString());

			// <company-code>
			addNode(mainTermNode, COMPANY_CODE, mainMember.getCompanyCode());
			
			// <department-code>
			addNode(mainTermNode, DEPARTMENT_CODE, mainMember
					.getDepartmentCode());
		}
	}

	public void createCompanyMember(XmlNode profileNode, User userModel) {

		CompanyMember[] companyMembers = userModel.getCompanyMembers();

		for (int i = 0; i < companyMembers.length; i++) {

			CompanyMember companyMember = companyMembers[i];

			// <company-member>
			XmlNode companyMemberNode = profileNode.addNode(COMPANY_MEMBER);

			// <company-code>
			addNode(companyMemberNode, COMPANY_CODE, companyMember
					.getCompanyCode());

			DepartmentMember[] departmentMembers = companyMember
					.getDepartmentMembers();
			for (int k = 0; k < departmentMembers.length; k++) {

				DepartmentMember departmentMember = departmentMembers[k];

				// <department-member>
				XmlNode departmentMemberNode = companyMemberNode
						.addNode(DEPARTMENT_MEMBER);

				// <department-code>
				addNode(departmentMemberNode, DEPARTMENT_CODE, departmentMember
						.getDepartmentCode());

				// <sort-key>
				addNode(departmentMemberNode, SORT_KEY, departmentMember
						.getSortKey());

				Term[] terms = departmentMember.getTerms();

				for (int m = 0; m < terms.length; m++) {

					Term term = terms[m];

					// <department-member-term>
					XmlNode departmentMemberTermNode = departmentMemberNode
							.addNode(DEPARTMENT_MEMBER_TERM);

					// <start-date>
					addNode(departmentMemberTermNode, START_DATE, term
							.getActionvationString());

					// <end-date>
					addNode(departmentMemberTermNode, END_DATE, term
							.getExpirationString());

					// <post-code>
					addNode(departmentMemberTermNode, POST_CODE,
							departmentMember.getPostCode(term));
				}
			}
		}
	}

	public void createPublicGroupSetMember(XmlNode profileNode, User userModel) {

		PublicGroupSetMember[] publicGroupSetMembers = userModel
				.getPublicGroupSetMembers();

		for (int i = 0; i < publicGroupSetMembers.length; i++) {

			PublicGroupSetMember publicGroupSetMember = publicGroupSetMembers[i];

			// <public-group-set-member>
			XmlNode publicGroupSetMemberNode = profileNode
					.addNode(PUBLIC_GROUP_SET_MEMBER);

			// <public-group-set-code>
			addNode(publicGroupSetMemberNode, PUBLIC_GROUP_SET_CODE,
					publicGroupSetMember.getPublicGroupSetCode());

			PublicGroupMember[] publicGroupMembers = publicGroupSetMember
					.getPublicGroupMembers();

			for (int k = 0; k < publicGroupMembers.length; k++) {

				PublicGroupMember publicGroupMember = publicGroupMembers[k];

				// <public-group-member>
				XmlNode publicGroupMemberNode = publicGroupSetMemberNode
						.addNode(PUBLIC_GROUP_MEMBER);

				// <public-group-code>
				addNode(publicGroupMemberNode, PUBLIC_GROUP_CODE,
						publicGroupMember.getPublicGroupCode());

				// <sort-key>
				addNode(publicGroupMemberNode, SORT_KEY, publicGroupMember
						.getSortKey());

				Term[] terms = publicGroupMember.getTerms();

				for (int m = 0; m < terms.length; m++) {

					Term term = terms[m];
					
					// <public-group-member-term>
					XmlNode publicGroupMemberTermNode = publicGroupMemberNode.addNode(PUBLIC_GROUP_MEMBER_TERM);
					
					// <start-date>
					addNode(publicGroupMemberTermNode, START_DATE, term
							.getActionvationString());

					// <end-date>
					addNode(publicGroupMemberTermNode, END_DATE, term
							.getExpirationString());
				}
			}
		}
	}
}
