package jp.co.intra_mart.tools.bm5.csv2xml.models.company;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import jp.co.intra_mart.tools.bm5.csv2xml.common.Const;
import jp.co.intra_mart.tools.bm5.csv2xml.models.common.ModelBuilderException;
import jp.co.intra_mart.tools.bm5.csv2xml.models.common.ValueConverter;
import jp.co.intra_mart.tools.bm5.csv2xml.models.common.Version;

/**
 * Ђ̃fłB
 * 
 * @since 5.1
 * @author INTRAMART
 * @version 1.0
 */
public class Company implements Serializable {
    /**
     * ЃR[h
     */
    private final String companyCode;
    /**
     * gDێ}bv
     */
    private Map departments = new HashMap();
    /**
     * gDێ郊Xg
     */
    private List departmentList = new ArrayList();
    /**
     * o[Wێ\[g}bv
     */
    private SortedMap versions = new TreeMap();
    /**
     * gD\̃[gvf
     */
    private Map rootNodes = new HashMap();

    /**
     * RXgN^
     * 
     * @param companyCode ЃR[h
     */
    public Company(String companyCode) {
        ValueConverter.checkNull(Const.COMPANY_CODE, companyCode);
        this.companyCode = companyCode;
    }
    
    /**
     * ЃR[h擾܂B
     * 
     * @return ЃR[h
     */
    public String getCompanyCode() {
        return this.companyCode;
    }
    
    /**
     * gDZbg̃[gvfԂ܂B
     * 
     * @return gD
     */
    public Department getRoot() {
       return getDepartment(getCompanyCode()); 
    }
    
    /**
     * gDݒ肵܂B
     * 
     * @param department gD
     */
    public void addDepartment(Department department) {
        if (existDepartment(department.getDepartmentCode())) {
            // łɓo^ς݂̑gD
            throw new ModelBuilderException("MW-003", new String[] {
                    getCompanyCode(), department.getDepartmentCode() });
        }
        
        this.departments.put(department.getDepartmentCode(), department);
        this.departmentList.add(department);
        department.setCompany(this);
    }

    /**
     * w肳ꂽgDR[hgDԂ܂B
     * 
     * @param departmentCode gDR[h
     * @return gD
     */
    public Department getDepartment(String departmentCode) {
        if (!this.existDepartment(departmentCode)) {
            // YgD݂Ȃ
            throw new ModelBuilderException("MW-004", new String[] { getCompanyCode(),
                    departmentCode });
        }
        return (Department) departments.get(departmentCode);
    }
    
    /**
     * w肳ꂽgDR[hɈvO[v݂邩ǂ
     * 
     * @param departmentCode gD
     * @return ݂ꍇtrueAȊOfalse
     */
    public boolean existDepartment(String departmentCode) {
        return departments.containsKey(departmentCode);
    }
    
    /**
     * Kw\o[Wǉ܂B
     * 
     * @param version o[W
     */
    public void addVersion(Version version) {
        
        if (this.versions.containsValue(version)) {
            // o[WR[h݂
            throw new ModelBuilderException("MW-021", new String[] {
                    getCompanyCode(), version.getVersionCode() });
        }
        
        try {
            if (this.versions.containsKey(version.getTerm())) {
                // Ԃ̃o[W񂪑݂
                throw new ModelBuilderException("MW-005", new String[] {
                        getCompanyCode(), version.getVersionCode(), version.getTerm().toString() });
            }
            
            this.versions.put(version.getTerm(), version);
        } catch (IllegalArgumentException e) {
            // ԂdȂo[W݂
            throw new ModelBuilderException("MW-006", new String[] {
                    getCompanyCode(), version.getVersionCode(), version.getTerm().toString() });
        }
    }
    
    /**
     * Kw\o[Ẅꗗ擾܂B
     * 
     * @return Kw\o[Ẅꗗ
     */
    public Version[] getVersions() {
        return (Version[]) versions.values().toArray(new Version[versions.size()]);
    }
    
    /**
     * gD̊Kw\ݒ肵܂B
     * 
     * @param versionCode o[WR[h
     * @param parentCode egDR[h
     * @param childCode qgDR[h
     */
    public void setStruct(String versionCode, String parentCode, String childCode) {
        
        // qgDɉЂݒ肳ꂽꍇ
        if (getCompanyCode().equals(childCode)) {
            // egDݒ肳Ăꍇ
            if (parentCode != null && parentCode.length() > 0) {
                // ЂqgDɐݒłȂ
                throw new ModelBuilderException("MW-020", getCompanyCode());
            }
            return;
        }
        
        if (!versions.containsValue(new Version(versionCode, null))) {
            // Yo[W񂪑݂Ȃ
            throw new ModelBuilderException("MW-007", new String[] {
                    getCompanyCode(), versionCode });
        }
        
        Department parent = getDepartment(parentCode);
        if (parent.getTerms().length == 0) {
            // egD̊ԉ񂪑݂Ȃ
            throw new ModelBuilderException("MW-022", new String[] {
                    getCompanyCode(), parent.getDepartmentCode() });
        }
        
        Department child = getDepartment(childCode);
        if (child.getTerms().length == 0) {
            // qgD̊ԉ񂪑݂Ȃ
            throw new ModelBuilderException("MW-023", new String[] {
                    getCompanyCode(), parent.getDepartmentCode() });
        }

        parent.addChild(versionCode, child);
        Department structRoot = child.setParent(versionCode, parent);

        // Џł͂Ȃꍇ͕ۑ
        if (!structRoot.equals(getRoot())) {
            if (!rootNodes.containsKey(versionCode)) {
                rootNodes.put(versionCode, new HashSet());
            }
            ((Set) rootNodes.get(versionCode)).add(structRoot);
        }
        
        // qvf[g폜
        if (rootNodes.containsKey(versionCode)) {
            ((Set) rootNodes.get(versionCode)).remove(child);
        }
    }
    
    /**
     * gD̈ꗗԂ܂B
     * 
     * @return gD̈ꗗ
     */
    public Department[] getDepartmentList() {
        return (Department[]) departmentList.toArray(new Department[departmentList.size()]);
    }
    
    /**
     * w肳ꂽgDԉĂ邩ǂ
     * 
     * @param departmentCode gDR[h
     * @return ԉĂꍇtrueAȊȌꍇfalse
     */
    public boolean isTerminable(String departmentCode) {
        if (existDepartment(departmentCode)) {
            return getDepartment(departmentCode).getTerms().length > 0;
        }
        return false;
    }

    /**
     * sȊKw\ɂgDԂ܂B
     * 
     * @return sȊKw\
     */
    public Map getInvalidStruct() {
        for (Iterator it = rootNodes.keySet().iterator(); it.hasNext(); ) {
            String versionCode = (String) it.next();
            if (((Set) rootNodes.get(versionCode)).isEmpty()) {
                // sȑgDȂo[W͍폜
                rootNodes.remove(versionCode);
            }
        }
        return rootNodes;
    }

    
    public int hashCode() {
        return getCompanyCode().hashCode();
    }
    
    public boolean equals(Object obj) {
        if (obj == null || !(obj instanceof Company)) {
            return false;    
        }
        return (this.hashCode() == obj.hashCode());
    }
}
