package jp.co.intra_mart.tools.bm5.csv2xml.models.public_group;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import jp.co.intra_mart.tools.bm5.csv2xml.common.Const;
import jp.co.intra_mart.tools.bm5.csv2xml.models.common.ModelBuilderException;
import jp.co.intra_mart.tools.bm5.csv2xml.models.common.ValueConverter;
import jp.co.intra_mart.tools.bm5.csv2xml.models.common.Version;

/**
 * pubNO[vZbg̃fłB
 * 
 * @since 5.1
 * @author INTRAMART
 * @version 1.0
 */
public class PublicGroupSet implements Serializable {
    /**
     * pubNO[vZbgR[h
     */
    private String publicGroupSetCode = null;
    /**
     * pubNO[vێ}bv
     */
    private Map groups = new HashMap();
    /**
     * pubNO[vێ郊Xg
     */
    private List groupList = new ArrayList();
    /**
     * o[Wێ\[g}bv
     */
    private SortedMap versions = new TreeMap();
    /**
     * gD\̃[gvf
     */
    private Map rootNodes = new HashMap();
    
    /**
     * RXgN^
     * 
     * @param publicGroupSetCode O[vZbgR[h
     */
    public PublicGroupSet(String publicGroupSetCode) {
        ValueConverter.checkNull(Const.PUBLIC_GROUP_SET_CODE, publicGroupSetCode);
        this.publicGroupSetCode = publicGroupSetCode;
    }
    
    /**
     * pubNO[vR[hԂ܂B
     * 
     * @return pubNO[vR[h
     */
    public String getPublicGroupSetCode() {
        return this.publicGroupSetCode;
    }
    
    /**
     * pubNO[vZbg̃[gvfԂ܂B
     * 
     * @return pubNO[v
     */
    public PublicGroup getRoot() {
       return getPublicGroup(getPublicGroupSetCode()); 
    }
    
    /**
     * pubNO[vݒ肵܂B
     * 
     * @param publicGroup pubNO[v
     */
    public void addPublicGroup(PublicGroup publicGroup) {
        
        if (existPublicGroup(publicGroup.getPublicGroupCode())) {
            // łɓo^ς݂̃O[v
            throw new ModelBuilderException("MW-203", new String[] {
                    getPublicGroupSetCode(), publicGroup.getPublicGroupCode() });
        }
        
        this.groups.put(publicGroup.getPublicGroupCode(), publicGroup);
        this.groupList.add(publicGroup);
        publicGroup.setPublicGroupSet(this);
    }

    /**
     * w肳ꂽpubNO[vR[hpubNO[vԂ܂B
     * 
     * @param publicGroupCode pubNO[vR[h
     * @return pubNO[v
     */
    public PublicGroup getPublicGroup(String publicGroupCode) {
        if (!this.existPublicGroup(publicGroupCode)) {
            // YO[v݂Ȃ
            throw new ModelBuilderException("MW-204", new String[] {
                    getPublicGroupSetCode(), publicGroupCode });
        }
        return (PublicGroup) groups.get(publicGroupCode);
    }
    
    /**
     * w肳ꂽpubNO[vR[hɈvO[v݂邩ǂ
     * 
     * @param publicGroupCode pubNO[v
     * @return ݂ꍇtrueAȊOfalse
     */
    public boolean existPublicGroup(String publicGroupCode) {
        return groups.containsKey(publicGroupCode);
    }
    
    /**
     * Kw\o[Wǉ܂B
     * 
     * @param version o[W
     */
    public void addVersion(Version version) {
        
        if (this.versions.containsValue(version)) {
            // o[WR[h݂
            throw new ModelBuilderException("MW-221", new String[] {
                    getPublicGroupSetCode(), version.getVersionCode() });
        }


        try {
            if (this.versions.containsKey(version.getTerm())) {
                // Ԃ̃o[W񂪑݂
                throw new ModelBuilderException("MW-205", new String[] {
                        getPublicGroupSetCode(), version.getVersionCode(),
                        version.getTerm().toString() });
            }

            this.versions.put(version.getTerm(), version);
        } catch (IllegalArgumentException e) {
            // ԂdȂo[W݂
            throw new ModelBuilderException("MW-206", new String[] {
                    getPublicGroupSetCode(), version.getVersionCode(),
                    version.getTerm().toString() });
        }
    }
    
    /**
     * Kw\o[Ẅꗗ擾܂B
     * 
     * @return Kw\o[Ẅꗗ
     */
    public Version[] getVersions() {
        return (Version[]) versions.values().toArray(new Version[versions.size()]);
    }
    
    /**
     * pubNO[v̊Kw\ݒ肵܂B
     * 
     * @param versionCode o[WR[h
     * @param parentCode epubNO[vR[h
     * @param childCode qpubNO[vR[h
     */
    public void setStruct(String versionCode, String parentCode, String childCode) {

        // qO[vɃpubNO[vZbgݒ肳ꂽꍇ
        if (getPublicGroupSetCode().equals(childCode)) {
            // epubNO[vݒ肳Ăꍇ
            if (parentCode != null && parentCode.length() > 0) {
                // pubNO[vZbgqO[vɐݒłȂ
                throw new ModelBuilderException("MW-220", getPublicGroupSetCode());
            }
            return;
        }

        // o[Wo^Ă邩ǂ
        if (!versions.containsValue(new Version(versionCode, null))) {
            // Yo[W񂪑݂Ȃ
            throw new ModelBuilderException("MW-207", new String[] {
                    getPublicGroupSetCode(), versionCode });
        }
        
        PublicGroup parent = getPublicGroup(parentCode);
        if (parent.getTerms().length == 0) {
            // egD̊ԉ񂪑݂Ȃ
            throw new ModelBuilderException("MW-222", new String[] {
                    getPublicGroupSetCode(), parent.getPublicGroupCode() });
        }
        
        PublicGroup child = getPublicGroup(childCode);
        if (child.getTerms().length == 0) {
            // qgD̊ԉ񂪑݂Ȃ
            throw new ModelBuilderException("MW-223", new String[] {
                    getPublicGroupSetCode(), parent.getPublicGroupCode() });
        }

        parent.addChild(versionCode, child);
        PublicGroup structRoot = child.setParent(versionCode, parent);
        
        // pubNO[vZbgłȂꍇ͕ۑ
        if (!structRoot.equals(getRoot())) {
            if (!rootNodes.containsKey(versionCode)) {
                rootNodes.put(versionCode, new HashSet());
            }
            ((Set) rootNodes.get(versionCode)).add(structRoot);
        }
        
        // qvf[g폜
        if (rootNodes.containsKey(versionCode)) {
            ((Set) rootNodes.get(versionCode)).remove(child);
        }
    }
    
    /**
     * pubNO[v̈ꗗԂ܂B
     * 
     * @return pubNO[v̈ꗗ
     */
    public PublicGroup[] getPublicGroupList() {
        return (PublicGroup[]) groupList.toArray(new PublicGroup[groupList.size()]);
    }
    
    /**
     * w肳ꂽpubNO[vԉĂ邩ǂ
     * 
     * @param publicGroupCode pubNO[vR[h
     * @return ԉĂꍇtrueAȊȌꍇfalse
     */
    public boolean isTerminableGroup(String publicGroupCode) {
        if (existPublicGroup(publicGroupCode)) {
            return getPublicGroup(publicGroupCode).getTerms().length > 0;
        }
        return false;
    }
    
    /**
     * sȊKw\ɂpubNO[vԂ܂B
     * 
     * @return sȊKw\
     */
    public Map getInvalidStruct() {
        for (Iterator it = rootNodes.keySet().iterator(); it.hasNext(); ) {
            String versionCode = (String) it.next();
            if (((Set) rootNodes.get(versionCode)).isEmpty()) {
                rootNodes.remove(versionCode);
            }
        }
        return rootNodes;
    }
    
    public int hashCode() {
        return getPublicGroupSetCode().hashCode();
    }
    
    public boolean equals(Object obj) {
        if (obj == null || !(obj instanceof PublicGroupSet)) {
            return false;    
        }
        return (this.hashCode() == obj.hashCode());
    }
}
